import os

# dictionary for subjects
SUBJECT_LONG_DICT = {
    'Mathe': 'M',
    'Deutsch': 'D',
    'Englisch': 'E',
    'Sport': 'S',
    'Französisch': 'F',
    'Technik': 'T'
}

# reversed dictionary for subjects
SUBJECT_SHORT_DICT = {short: long for long, short in SUBJECT_LONG_DICT.items()}

base_dir = os.path.split(os.getcwd())[0]
data_dir = os.path.join(base_dir, 'backup_data')


class Class:
    def __init__(self, level: int, name: str, subjects: dict):
        self.level = level
        self.name = name
        self.subjects = subjects


    def __str__(self):
        return f'Klasse:\t {self.level}{self.name}\n' \
               f'Fächer:\t {self._get_subjects()}'


    def list_it(self):
        return [self.level, self.name, ';'.join(self.subjects_to_list())]


    def subjects_to_list(self):
        liste = []
        for i in self.subjects:
            liste.append(str(i + f'-{self.subjects[i][0]}-{self.subjects[i][1]}'))
        return liste


    def _get_subjects(self):
        text = ''
        for i in self.subjects:
            text += i + f' ({self.subjects[i][0]} Stunden) bei {self.subjects[i][1]}\n\t\t '
        return text


    def get_fullname(self):
        return f'{self.level}{self.name}'


    def hours_missing(self):
        for i in self.subjects:
            if self.subjects[i][1] == 'null':
                return True
        return False


    def list_teachers(self):
        teachers = []
        for i in self.subjects:
            if self.subjects[i][1] not in teachers and self.subjects[i][1] != 'null':
                teachers.append(self.subjects[i][1])
        return teachers


class Teacher:
    def __init__(self, name: str, short: str, hours: int, subjects: list):
        assert hours >= 0, f'{name} hat weniger als 0 Stunden ({hours}).'
        self.name = name
        self.short = short
        self.hours = hours
        self.subjects = subjects
        self.hours_left = hours


    def __str__(self):
        subs = {}
        for s in self.subjects:
            s = SUBJECT_SHORT_DICT[s]
            subs[s] = 0
        for c in AllClasses.classes:
            for s in c.subjects:
                if SUBJECT_SHORT_DICT[s] in subs:
                    if c.subjects[s][1] == self.short:
                        subs[SUBJECT_SHORT_DICT[s]] += int(c.subjects[s][0])
        text = f'Name:\t {self.name} ({self.short})\n' \
               f'Fächer:\t {_build_string_from_dict(subs)}\n' \
               f'Stunden: {self.hours} ({self._get_hours_left()}h übrig)'
        return text


    def __repr__(self):
        return f'Teacher({self.name}, {self.short}, {self.hours}, {self.subjects})'


    def list_it(self):
        return [self.name, self.short, str(self.hours), ';'.join(self.subjects)]


    def _get_hours_left(self):
        teached_hours = []
        for c in AllClasses.classes:
            for s in c.subjects:
                if c.subjects[s][1] == self.short:
                    teached_hours.append(int(c.subjects[s][0]))
        return self.hours - sum(teached_hours)



class AllTeachers:
    teachers = []
    backup = []
    filename = 'lehrer.csv'

    def read_data(self, path, name=None, additional=False):
        if not additional:
            self.teachers = []
        if name:
            path, name = os.path.split(path)
            build_data(name, path)
        else:
            build_data(self.filename, path)


    def save_data(self, location):
        save_data(self.filename, location)


    def _list_teacher_hours(self):
        list = []
        for t in self.teachers:
            subs = {}
            for s in t.subjects:
                subs[s] = 0
            for c in AllClasses.classes:
                for s in c.subjects:
                    if s in subs:
                        if c.subjects[s][1] == t.short:
                            subs[s] += int(c.subjects[s][0])
            list.append(f'{t.name}\t({t.short})\t-\t[{_build_string_from_dict(subs)}]')
        return list


    def add_teacher(self, name, short, hours, subjects):
        new = Teacher(name, short, hours, subjects)
        for i in self.teachers:
            if i.short == new.short:
                return False
        self.teachers.append(new)
        print(new)
        return True


class AllClasses:
    classes = []
    backup = []
    filename = 'klassen.csv'

    def read_data(self, path, name=None, additional=False):
        if not additional:
            self.classes = []
        if name:
            path, name = os.path.split(path)
            build_data(name, path)
        else:
            build_data(self.filename, path)

    def save_data(self, location):
        save_data(self.filename, location)


def _build_string_from_dict(subs: dict):
    text = ""
    for i in subs:
        text += f'{i}: {subs[i]}h, '
    return text[0:-2]


def build_object(line: list, typ: str) -> bool:
    if typ == 'lehrer.csv':
        name, short, hours, subjects = line[0], line[1], int(line[2]), line[3]
        new = Teacher(name, short, hours, subjects)
        for t in AllTeachers.teachers:
            if t.short == new.short:
                print('skiped')
                return False
        AllTeachers.teachers.append(new)
        return True
    elif typ == 'klassen.csv':
        level, name, subjects = line[0], line[1], line[2]
        AllClasses.classes.append(Class(level, name, subjects))
        return True
    elif typ == 'fach':
        return True
    return False


def build_data(filename: str, path=data_dir):
    with open(os.path.join(path, filename), 'r') as file:
        lines = file.readlines()[1:]
    for line in lines:
        line = line.strip().split(',')
        if filename == 'lehrer.csv':
            line[-1] = line[-1].split(';')
        elif filename == 'klassen.csv':
            line[-1] = line[-1].split(';') # 'S-2-null'
            new = {}
            for fach in line[-1]:
                fach = fach.split('-') # ['S', '2', 'null']
                fach[1] = int(fach[1])
                new[fach[0]] = fach[1:]
            line[-1] = new
        build_object(line, filename)


def save_data(filename: str, path=data_dir):
    path = os.path.join(path, filename)
    try:
        file = open(path, 'w')
        if filename == 'lehrer.csv':
            print('name,kürzel,stunden,fächer', file=file)
            for obj in AllTeachers.teachers:
                print(",".join(obj.list_it()), file=file)
        elif filename == 'klassen.csv':
            print('klassenstufe,name,fächer(fach-stundenzahl-lehrer)', file=file)
            for obj in AllClasses.classes:
                print(",".join(obj.list_it()), file=file)
        file.close()
    except TypeError as error:
        print(error)


def add_teacher():
    print('teacher added')


def add_class():
    print('class added')


if __name__ == '__main__':
    AllTeachers().read_data(data_dir)


from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='deputat',
    version='0.2.8',
    description='deputat overview',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/lfreist/deputat",
    author='lfreist',
    author_email='freist.leon@gmx.de',
    packages=['deputat', 'deputat/GUI/', 'deputat/GUI/pictures/'],
    install_requires=[
              'PyQt5',
              ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        ],
)




import os
import sys
from deputat import settings

sys.path.insert(1, settings.base_dir())

from deputat.deputat import AllTeachers, SUBJECT_LONG_DICT

from PyQt5.QtWidgets import (QDialog, QDialogButtonBox, QFormLayout, QGroupBox, QLabel,
                             QLineEdit, QVBoxLayout, QMessageBox, QCheckBox, QSlider)

from PyQt5.QtCore import Qt


class AddTeacherPopUp(QDialog):
    icon_path = settings.icon_dir()
    NumGridRows = 3
    NumButtons = 4

    def __init__(self, name, parent=None):
        super().__init__(parent)
        self.create_form_GB()

        button_box = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
        button_box.accepted.connect(self.add)
        button_box.rejected.connect(self.reject)

        main_layout = QVBoxLayout()
        main_layout.addWidget(self.formGroupBox)
        main_layout.addWidget(button_box)
        self.setLayout(main_layout)

        self.setWindowTitle(name)

    def create_form_GB(self):
        self.formGroupBox = QGroupBox("Informationen")
        self.layout = QFormLayout()
        self.name, self.short = QLineEdit(), QLineEdit()
        self.subjects = [QCheckBox(s) for s in SUBJECT_LONG_DICT]
        self.layout.addRow(QLabel("Name:"), self.name)
        self.layout.addRow(QLabel("Kürzel:"), self.short)
        self.layout.addRow(QLabel("Fächer:"), self.subjects[0])
        for s in self.subjects[1:]:
            self.layout.addRow(QLabel(''), s)
        self.hours = QSlider(Qt.Horizontal)
        self.hours.setMinimum(1)
        self.hours.setMaximum(28)
        self.hours.setValue(22)
        self.hours.setTickPosition(QSlider.TicksAbove)
        self.hours.setTickInterval(1)
        self.hours.valueChanged.connect(self.update_label)
        self.hours_edit = QLineEdit()
        self.hours_edit.textChanged.connect(self.update_hours)
        self.layout.addRow(self.hours_edit, self.hours)
        self.formGroupBox.setLayout(self.layout)


    def update_label(self):
        self.hours_edit.setText(str(self.hours.value()))


    def update_hours(self):
        try:
            self.hours.setValue(int(self.hours_edit.text()))
        except (TypeError, ValueError):
            pass

    def add(self):
        subs = []
        index = []
        for i, o in enumerate(self.subjects):
            if o.isChecked():
                index.append(i)
        for i, o in enumerate(SUBJECT_LONG_DICT):
            if i in index:
                subs.append(SUBJECT_LONG_DICT[o])
        if not subs or not self.name.text() or not self.short.text() or not self.hours.value():
            return
        AllTeachers().add_teacher(self.name.text(), self.short.text(), int(self.hours.value()), subs)
        self.accept()



class QuitPopUp(QMessageBox):

    def __init__(self, name, parent=None):
        super().__init__(parent)
        self.setIcon(QMessageBox.Warning)
        self.setText("Ohne speichern verlassen?")
        self.setWindowTitle("Exit")
        self.setDetailedText("Es scheint, als wäre etwas verändert worden."
                             "Wird nicht gespeichert, gehen alle Änderungen verloren!\n"
                             "Diese Warnung kann jedoch auch fälschlicherweise angezeigt werden,"
                             "wenn Sie Änderungen manuell rückgängig gemacht werden...")
        self.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
        self.setDefaultButton(QMessageBox.No)

        self._close = None

        returnValue = self.exec()
        if returnValue == QMessageBox.Yes:
            self._close = True
        if returnValue == QMessageBox.No:
            self._close = False


    def get(self):
        return self._close
